c-----
c     program for compliance
c     Two options
c     -HW water depth
c        the solid model starts at base of fluid
c        the phase velocity is from the water gravity wave
c     -C  phase velocity
c        the model is solid and compliance is at the top of the
c        model
c     -F  freq - eventually this will be a generated array
c     -M  model
c
c     output of program
c     compliance 
c     partial derivatives for each layer obtained by numerical
c         perturbation of the model
c-----
c       Changes
c       
c     03 March 2023 patched ST/f4a on to the original compliance.f
c       however this is inherently isotropic inspite of the face
c       that the parameters stored are TI - this is becuase the f4a was
c       generalized to tbe TI
c-----
        implicit none
c-----
c       LIN I*4 - logical unit for standard input
c       LOT I*4 - logical unit for standard output
c       LER I*4 - logical unit for error output
c-----
        integer LIN, LOT, LER
        parameter (LIN=5,LOT=6,LER=0)
c-----
        integer NL
        parameter (NL=200)
c-----
c     working model
c-----
        common/timodel/d(NL),TA(NL),TC(NL),TL(NL),TN(NL),TF(NL),
     1      TRho(NL),
     2      qa(NL),qb(NL),etap(NL),etas(NL),
     3      frefp(NL), frefs(NL)
        real d,TA,TC,TN,TL,TF,TRho,qa,qb,etap,etas,frefp,frefs
        common/depref/refdep
        real refdep
c-----
c     original model
c-----
        common/otimodel/od(NL),oTA(NL),oTC(NL),oTL(NL),oTN(NL),oTF(NL),
     1      oTRho(NL),
     2      oqa(NL),oqb(NL),oetap(NL),oetas(NL),
     3      ofrefp(NL), ofrefs(NL)
        real od,oTA,oTC,oTN,oTL,oTF,oTRho,oqa,oqb,oetap,
     1        oetas,ofrefp,ofrefs
        common/odepref/orefdep
        real orefdep

        integer mmax,mode
c-----
c       mname   C*80    - name of the model file
c       dfile   C*80    - name of the distance file
c       ext L   - logical variable to see if the model file 
c       ierr    I*4 - 0 model file read in correctly
c-----
c       mmax    I*4 - number of layers in the model, last layer is
c                    halfspace
c-----
        character title*80
        logical ext
        integer*4 iunit, iiso, iflsph
        integer idimen, icnvel, ierr

c-----
c     command line options
c-----
      real cvel , freq, depth, g
      logical seafloor
      character mname*80
c-----
c     other variables
c-----
      integer l, i
      integer lgstr
      real*8 cmpz, cmpr
c-----
c     kernels
c-----
c     kkappa, kmu, krho for (kappa,rho,rho)
c     ka, kb, kr        for (ka   ,kb ,kr )
c-----
      real kkappa, kmu, krho, ka, kb, kr
      real kappa, mu, lambda
      real oa, ob, orho
      real*8  tcmpzp,tcmprp
      real*8  tcmpzm,tcmprm
      real p, dp
      real*8 eta, deta
      real dz
      real zneg, zplus
      real eps

      REAL*8 DEBUG

      real*8  tcmpz,tcmpr
        real wa, wb, wrho
c-----
c     parse the command line
c-----
      call gcmdln(freq,cvel,depth,seafloor,mname,g)

      write(LOT,*)'Freq  (hz)   :',freq
      write(LOT,*)'c     (m/s)  :',cvel
      write(LOT,*)'seafloor     :',seafloor
      if(seafloor)then
           WRITE(LOT,*)'g   (m/s^2)  :', g
           WRITE(LOT,*)'depth   (m)  :', depth
      endif
c-----
c     get the model
c-----
c-----
c       get the earth model
c-----
        l = lgstr(mname)

        write(LOT,*)'Model name: ',mname(1:l)
        call getmod(1,mname,mmax,title,iunit,iiso,iflsph,
     1      idimen,icnvel,ierr,.true.)
c-----
c      save the parameters of the initial model
c-----
        do i=1,mmax
           oTrho(i) = Trho(i)
           oTA(i) = TA(i)
           oTC(i) = TC(i)
           oTF(i) = TF(i)
           oTL(i) = TL(i)
           oTN(i) = TN(i)
           od(i)  = d(i)
       enddo

c-----
c      get compliance for a model
c      also form the inversion kernels
c      compliance = Uz/Pzz
c      eta = compliance^2
c
c      kernel = (d eta/eta) (p/dp) (1/dz)
c
c      however  (d eta/deta) = 2 (d complinvr)/compliance
c-----
c-----
        call docpmod(mmax)
        call docomp(cmpz,cmpr,freq,cvel,mmax)
        WRITE(LOT,*)'COMPZ    :',CMPZ
        if(seafloor)then
        WRITE(LOT,*)'COMPR    :',CMPR
        else
        WRITE(LOT,*)'TILT     :',g*CMPZ/(cvel*6.28318*freq)
        endif
c-----
c       now get the partials
c       Tanimoto Wang kernels are
c       K = (Delta eta/eta )(p/Delta p)(1/Delta z)
c         where eta = cmpz**2
c               p   = kappa, mu, rho 
c         or        = alpha, beta, rho
c           (note changing mu changes the alpha of the model
c            and changing the rho changes the alpha and beta of of the first)
c-----
       write(LOT,'(a,a)') 
     1    'ZZ    I   Z(mid)(m)        Ka         Kb          Kr',
     2    '        Kkappa       Kmu         Krho'
       write(LOT,'(a,a)') 
     1    'XX    I   Z(mid)(m)        Ka         Kb          Kr',
     2    '        Kkappa       Kmu         Krho'
        zneg = 0.0
        do i = 1,mmax
          zplus = zneg + d(i)
          dz = d(i)
          call docpmod(mmax)
c-----
c         Each call to docpmod returns the orginal model in SI units
c         since this is the unperturbed model
c         save the velocities, deptity and Lame coefficients
c-----
          oa = sqrt(TC(i)/Trho(i))
          ob = sqrt(TL(i)/Trho(i))
          orho = Trho(i)
          mu = orho*ob*ob
          kappa = orho*oa*oa - 4.*mu/3.
          call docomp(cmpz,cmpr,freq,cvel,mmax)
          eps = 0.01
c-----
c       do a centered sided partial
c-----
c-----
c    Get K_\alpha K_\beta K_\rho
c       alpha
c-----
          call docpmod(mmax)
          p = oa
          dp = eps*p
          TA(i) = Trho(i)*(p + dp)**2
          TC(i) = Trho(i)*(p + dp)**2
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzp,tcmprp,freq,cvel,mmax)
          eta  = cmpz**2
          call docpmod(mmax)
          TA(i) = Trho(i)*(p - dp)**2
          TC(i) = Trho(i)*(p - dp)**2
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzm,tcmprm,freq,cvel,mmax)
C         deta = (tcmpzp - tcmpzm)*(tcmpzp + tcmpzm)/eta
          deta = (tcmpzp - tcmpzm)*(tcmpzp + tcmpzm)/eta
          ka = deta*(p/(2*dp))/dz
c-----
c       beta
c-----
          call docpmod(mmax)
          p = ob
          dp = eps*p
          TL(i) = Trho(i)*(p+dp)**2
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzp,tcmprp,freq,cvel,mmax)
          eta  = cmpz**2
          call docpmod(mmax)
          TL(i) = Trho(i)*(p-dp)**2
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzm,tcmprm,freq,cvel,mmax)
C         deta = (tcmpzp - tcmpzm)*(tcmpzp + tcmpzm)/eta
          deta = 2*(tcmpzp - tcmpzm)/cmpz
          kb = deta*(p/(2*dp))/dz
c-----
c       rho
c       Note if the densities change, then so to the TA TC TF TL
c-----
          call docpmod(mmax)
          p = orho
          dp = eps*p
          Trho(i) = p + dp
          TA(i) = Trho(i)*oa*oa
          TC(i) = Trho(i)*oa*oa
          TL(i) = Trho(i)*ob*ob
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzp,tcmprp,freq,cvel,mmax)
          eta  = cmpz**2
          call docpmod(mmax)
          Trho(i) = p - dp
          TA(i) = Trho(i)*oa*oa
          TC(i) = Trho(i)*oa*oa
          TL(i) = Trho(i)*ob*ob
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzm,tcmprm,freq,cvel,mmax)
C         deta = (tcmpzp - tcmpzm)*(tcmpzp + tcmpzm)/eta
          deta = 2*(tcmpzp - tcmpzm)/cmpz
          kr = deta*(p/(2*dp))/dz
c-----
c    Get K_\kappa K_\mu K_\rho
c       kappa
c    OK be careful about converting to A C F from kappa
c-----
          call docpmod(mmax)
          p = kappa
          dp = eps*p
          wa = sqrt( (p+dp +4.*mu/3. )/orho)
          wb = ob
          TA(i) = Trho(i)*wa*wa
          TC(i) = Trho(i)*wa*wa
          TL(i) = Trho(i)*wb*wb
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzp,tcmprp,freq,cvel,mmax)
          eta  = cmpz**2
          call docpmod(mmax)
          wa = sqrt( (p-dp +4.*mu/3. )/orho)
          wb = ob
          TA(i) = Trho(i)*wa*wa
          TC(i) = Trho(i)*wa*wa
          TL(i) = Trho(i)*wb*wb
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzm,tcmprm,freq,cvel,mmax)
C         deta = (tcmpzp - tcmpzm)*(tcmpzp + tcmpzm)/eta
          deta = 2*(tcmpzp - tcmpzm)/cmpz
          kkappa = deta*(p/(2*dp))/dz
c-----
c       mu
c-----
          call docpmod(mmax)
          p = mu
          dp = eps*p
          wa = sqrt(( kappa +4.*(p+dp)/3. )/orho)
          wb = sqrt( (p+dp)/orho)
          TA(i) = Trho(i)*wa*wa
          TC(i) = Trho(i)*wa*wa
          TL(i) = Trho(i)*wb*wb
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzp,tcmprp,freq,cvel,mmax)
          eta  = cmpz**2
          call docpmod(mmax)
          wa = sqrt(( kappa +4.*(p-dp)/3. )/orho)
          wb = sqrt( (p-dp)/orho)
          TA(i) = Trho(i)*wa*wa
          TC(i) = Trho(i)*wa*wa
          TL(i) = Trho(i)*wb*wb
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzm,tcmprm,freq,cvel,mmax)
C         deta = (tcmpzp - tcmpzm)*(tcmpzp + tcmpzm)/eta
          deta = 2*(tcmpzp - tcmpzm)/cmpz
          kmu = deta*(p/(2*dp))/dz
c-----
c       rho
c-----
          call docpmod(mmax)
          p = orho
          dp = eps*p
          wb = sqrt(mu/(p+dp))
          wa = sqrt((kappa+4*mu/3.)/(p+dp))
          Trho(i) = p + dp
          TA(i) = Trho(i)*wa*wa
          TC(i) = Trho(i)*wa*wa
          TL(i) = Trho(i)*wb*wb
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzp,tcmprp,freq,cvel,mmax)
          eta  = cmpz**2
          call docpmod(mmax)
          Trho(i) = p - dp
          wb = sqrt(mu/(p-dp))
          wa = sqrt((kappa+4*mu/3.)/(p-dp))
          TA(i) = Trho(i)*wa*wa
          TC(i) = Trho(i)*wa*wa
          TL(i) = Trho(i)*wb*wb
          TF(i) = TA(i) - 2*TL(i)
          call docomp(tcmpzm,tcmprm,freq,cvel,mmax)
C         deta = (tcmpzp - tcmpzm)*(tcmpzp + tcmpzm)/eta
          deta = 2*(tcmpzp - tcmpzm)/cmpz
          krho = deta*(p/(2*dp))/dz
c-----
c       output
c-----
         write(LOT,1)i,zneg+0.5*d(i),ka,kb,kr,kkappa,kmu,krho
    1    format('ZZ',i5,7f12.5)
         krho = kr -0.5*(ka+kb)
         kkappa =(0.5- (2*ob**2)/(3*oa**2))*ka
         kmu = (2*ob**2)/(3*oa**2)*ka +0.5*kb
         write(LOT,2)i,zneg+0.5*d(i),ka,kb,kr,kkappa,kmu,krho
    2    format('XX',i5,7f12.5)
         zneg = zplus
        enddo
       

        end

        subroutine docpmod(mmax)
c-----
c       copy the model
c       also convert from MIXED to SI units
c-----
        implicit none
        integer mmax
c-----
c     working model
c-----
        integer NL
        parameter (NL=200)
        common/timodel/d(NL),TA(NL),TC(NL),TL(NL),TN(NL),TF(NL),
     1      TRho(NL),
     2      qa(NL),qb(NL),etap(NL),etas(NL),
     3      frefp(NL), frefs(NL)
        real d,TA,TC,TN,TL,TF,TRho,qa,qb,etap,etas,frefp,frefs
        common/depref/refdep
        real refdep
c-----
c     original model
c-----
        common/otimodel/od(NL),oTA(NL),oTC(NL),oTL(NL),oTN(NL),oTF(NL),
     1      oTRho(NL),
     2      oqa(NL),oqb(NL),oetap(NL),oetas(NL),
     3      ofrefp(NL), ofrefs(NL)
        real od,oTA,oTC,oTN,oTL,oTF,oTRho,oqa,oqb,oetap,
     1        oetas,ofrefp,ofrefs
        common/odepref/orefdep
        real orefdep


        common/water/iwat(NL)
        integer iwat

        integer i
c-----
c       from MIXED to SI
c       density(kg/m^3) = 1000 density(gm/cm^3)
c       distance(m) = 1000 distance(km)
c       Elastic = density velocity^2
c       TA(kg/m^3 m/s m/s) = 10^3 (gm/cm^3) 10^3 (km/s) 10^3(km/s)
c-----
        do i=1,mmax
            d(i)    = od(i)   *1.0e+3
            TA(i)   = oTA(i)  *1.0e+9
            TC(i)   = oTC(i)  *1.0e+9
            TF(i)   = oTF(i)  *1.0e+9
            TN(i)   = oTN(i)  *1.0e+9
            TL(i)   = oTL(i)  *1.0e+9
            Trho(i) = oTrho(i)*1000.0
            iwat(i) = 0
        enddo
        return
        end

        subroutine gcmdln(freq,c,depth,seafloor,mname,g)
        implicit none
        real freq, c,depth
        logical seafloor
        character mname*(*)
c-----
c       freq - frequency for complance
c       c    - phase velocity always returned
c              either given on command line or
c              computed on the basis of the water thickness
c       seafloor - .true. computation is for sea floor compliance
c              this could be finessed, e.g., only done of model has
c              fluid layers overlying a solid in which case the
c              depth of the water column is given in the model
c              In this case the seafloor is not really used
c              and the depth is not an input
c       mname - name of the model
c------
       character instr*80
       integer mnmarg
       integer nmarg, i
       real  g
       logical ext

       nmarg = mnmarg()
       i = 0
       c = -1
       depth = -1
       freq = -1
       g = 9.8
       seafloor = .false.
 1000  continue
           i = i + 1
           if (i.gt.nmarg)go to 2000
           call mgtarg(i,instr)
           if(instr(1:2).eq.'-M')then
              i = i + 1
              call mgtarg(i,mname)
           else if(instr(1:2).eq.'-H')then
              i = i + 1
              call mgtarg(i,instr)
              read(instr,'(bn,f20.0)')depth
              seafloor = .true.
           else if(instr(1:2).eq.'-C')then
              i = i + 1
              call mgtarg(i,instr)
              read(instr,'(bn,f20.0)')c
           else if(instr(1:2).eq.'-F')then
              i = i + 1
              call mgtarg(i,instr)
              read(instr,'(bn,f20.0)')freq
           else if(instr(1:2).eq.'-G')then
              i = i + 1
              call mgtarg(i,instr)
              read(instr,'(bn,f20.0)')g
           else if(instr(1:2) .eq. '-h')then
               call usage(' ')
           endif
       go to 1000
 2000  continue
c-----
c       tests
c-----
c       get the earth model
c-----
        inquire(file=mname,exist=ext)
        if(.not. ext)call usage('Model file does not exist')
c-----
c       depth and c
c-----
        if(depth .lt.0 .and . c.lt.0)then
          call usage('Neither water depth or phase velocity given')
        endif
c-----
c       frequency
c------
        if(freq.lt.0.0)then
          call usage('Frequency not speciied')
        endif
c-----
c       if depth given, then derive the pase velocity
c-----
        if(depth.gt.0)then
             call getcvel(depth,c,g,freq)
        endif

        return 
        end

        subroutine usage(msg)
        implicit none
        character msg
c-----
c       LER I*4 - logical unit for error output
c-----
        integer LER
        parameter (LER=0)
        integer lgstr
        integer ls
        ls = lgstr(msg)
        if(ls.gt.0)then
           WRITE(LER,*)msg
        endif
        write(LER,*)'Usage: tcompliance.saito -M model ',
     1      '[-C cvel | -H depth] ',
     1  ' -F freq [-h]'
        write(LER,*)
     1  '  -M model   (default none) name of model file'
        write(LER,*)
     1  '  -C cvel    (default none) phase velocity (m/s)'
        write(LER,*)
     1  '  -H depth   (default none) water depth (m)'
        write(LER,*)
     1  '  -F freq    (default none) frequency'
        write(LER,*)
     1  '    If -C is invoked it is assumed that measurements are at',
     2  ' surface of solid Earth'
        write(LER,*)
     1  '    If -H is invoked seafloor compliance is computed and ',
     2  ' dept  is the height of the water column'
        write(LER,*)
     1  '  -h        (default none )  this help message '
        WRITE(LER,*)'Note for the water problem, the model is all solid'
        WRITE(LER,*)'since there is no propagation in fluid. The fluid'
        WRITE(LER,*)'pressure is given theoretically'
        stop
        end
       
        
        subroutine getcvel(depth,c,g,freq)
        implicit none
        real depth,c,g,freq

        real u, per
c-----
c       get the phase velocity of the gravity wave
c       for a given frequency
c
c       omega^2 = g k tanh( k H)
c-----
        per = 1.0/freq
        call getroot(per,g,depth,c,u)
        return
        end


       subroutine getroot(per,g,h,c,u)
       real per, g, h, c
       real fk, dfdk,k
       freq = 1.0/per
c-----
c      start search between 1 and 1.5 sqrt(gH)
c-----
       cmin = 1.0
       cmax = 1.5*sqrt(g*h)
       fmin =  fk(freq,cmin,g,h,u)
       fmax =  fk(freq,cmax,g,h,u)
       do i=1,20
           cmid = 0.5*(cmin+cmax)
           fmid =  fk(freq,cmid,g,h,u)
C          WRITE(6,*)cmid,fmid,cmax,fmax,cmin,fmin
           if(sign(1.0,fmid).eq.sign(1.0,fmax))then
                   cmax = cmid
                   fmax = fmid
           else
                   cmin = cmid
                   fmin = fmid
           endif
       enddo
           c = 0.5*(cmin+cmax)
       return
       end

       function dfdk(freq,c,g,h)
       real freq,c,w,k,g,h
       real tahh
       w = 6.2831853*freq
       k = w/c
       dfdk =  - g*tanh(k*h) -g*k*(1.-tanh(k*h))*h
       return
       end

       function fk(freq,c,g,h,u)
       real freq,c,w,k,g,h
       real tanh
       real sinh
       w = 6.2831853*freq
       k = w/c
       fk = w*w - g*k*tanh(k*h)
       u = 0.5*c*(1 + (2*k*h)/sinh(2*k*h))
       return
       end

       function sinh(x)
           fac = exp(x)
           sinh = ( fac - 1./fac)/2.
       return
       end


       function tanh(x)
       fac1 = 2.0*x
       if(fac1.gt.50.0)then
           tanh = 1.0
       else
           efac = exp(-fac1)
           tanh = ( 1 - efac)/(1+efac)
       endif
       return
       end


       subroutine docomp(cmpz,cmpr,freq,cvel,mmax)
       real*8 cmpz,cmpr
       real*4 freq,cvel

        integer NL
        parameter (NL=200)
        common/cy/Y12(NL),Y13(NL),Y14(NL),Y23(NL),Y24(NL),Y34(NL)
        complex Y12,Y13,Y14,Y23,Y24,Y34
        common/norm/YIJnorm(NL),Ynorm(NL)

        real dltar
        real dltarret

        dltarret =  dltar(freq,cvel,mmax)
C       WRITE(6,*)Y14(1),Y24(1),Y34(1)
       cmpr =  Y14(1)/Y34(1)
       cmpz =  Y24(1)/Y34(1)
c-----
c      the above is displacement positive down and away
c      to traction.  Multiply by -1 to get displaement to pressure
c-----
       cmpr = - cmpr
       cmpz = - cmpz
       return
       end

c------
c      this is the transition to the Takeuchi and Saito part
c------

c-----
c     this is from f4a.f
c-----

        real function dltar(freq,c,mmax)
        implicit none
        real freq, c
        integer mmax
        complex Y(5)

        integer lyr
        integer i

        integer NL
        parameter (NL=200)
        common/cy/Y12(NL),Y13(NL),Y14(NL),Y23(NL),Y24(NL),Y34(NL)
        complex Y12,Y13,Y14,Y23,Y24,Y34
        common/norm/YIJnorm(NL),Ynorm(NL)
        real YIJnorm, Ynorm
        real vnorm
        
        call hinit(Y,freq,c,mmax)
        call norm5(Y,vnorm)
        Y12(mmax) =  Y(1)
        Y13(mmax) =  Y(2)
        Y14(mmax) =  Y(3)
        Y23(mmax) = -Y(3)
        Y24(mmax) =  Y(4)
        Y34(mmax) =  Y(5)
C       WRITE(6,*)'Halfspace Y(5)=',Y(5)
c-----
c       Runge-Kutta integration
c-----
        do lyr=mmax-1,1,-1
            call rkutta5(Y,freq,c,lyr)
            call norm5(Y,vnorm)
            Y12(lyr) =  Y(1)
            Y13(lyr) =  Y(2)
            Y14(lyr) =  Y(3)
            Y23(lyr) = -Y(3)
            Y24(lyr) =  Y(4)
            Y34(lyr) =  Y(5)
            YIJnorm(lyr) = vnorm + YIJnorm(lyr+1)
        enddo
C       do i=1,5
C          if(i.lt.5)then
C               write(6,*)i, Y(i)
C          else
C               write(6,*)i, Y(i), Y(3)/Y(4)
C          endif
C       enddo
C       do i=1,5
C              write(6,*)lyr,i, Y(i)
C       enddo
        dltar = Y(5)

        return
        end

        subroutine rkutta5(Y,freq,cvel,lyr)
        complex Y(5)
        real freq, cvel
        integer lyr

        real amat(5,5)
        complex K1(5), K2(5), K3(5), K4(5)
        complex Ytemp(5)
        real hh
        integer n, nsub
        real dz 
        integer i

C       WRITE(6,*)'lyr=',lyr,Y
c-----
c       the ODE to be solved is
c       dY/dz = A Y where A is constant in a layer
c-----
        call dofill5(amat,lyr,freq,cvel,hh)
        nsub = 100
        dz = -hh/nsub
c-----
c       split layer into 5 sublayers
c-----
        do n=1,nsub
            call cmult5(Y,K1,amat)
            do i=1,5
               Ytemp(i) = Y(i) + 0.5*dz*K1(i)
            enddo
            call cmult5(Ytemp,K2,amat)
            do i=1,5
               Ytemp(i) = Y(i) + 0.5*dz*K2(i)
            enddo
            call cmult5(Ytemp,K3,amat)
            do i=1,5
               Ytemp(i) = Y(i) + dz*K3(i)
            enddo
            call cmult5(Ytemp,K4,amat)
            do i=1,5
               Y(i) = Y(i) +(k1(i)+2*k2(i)+2*k3(i)+k4(i))*dz/6.
            enddo
        enddo
        return
        end
    
        subroutine norm5(Y,vnorm)
        complex Y(5)
        real vnorm

        real eval
        integer i
    
c-----
c       apply normalization
c-----
        eval = 0.0
        do i=1,5
           if(cabs(Y(i)).gt.eval)then
              eval = cabs(Y(i))
           endif
        enddo
        do i=1,5
            Y(i) = Y(i) / eval
        enddo
        vnorm = alog(eval)
               
        return
        end

        subroutine cmult5(Y0,Y1,A)
        implicit none
        complex Y1(5), Y0(5)
        real A(5,5)
        integer i,j
        do i=1,5
           Y1(i) = cmplx(0.0,0.0)
           do j=1,5
              Y1(i) = Y1(i) + A(i,j)*Y0(j)
           enddo
        enddo
        return 
        end

        subroutine cmult4(Y0,Y1,A)
        implicit none
        real Y1(4), Y0(4)
        real A(4,4)
        integer i,j
        do i=1,4
           Y1(i) = cmplx(0.0,0.0)
           do j=1,4
              Y1(i) = Y1(i) + A(i,j)*Y0(j)
           enddo
        enddo
        return 
        end
   
        
     

        subroutine hinit(Y,freq,cvel,mmax)
        implicit none
c-----
c       assume halfspace is isotropic
c-----
        complex Y(5)
        real freq, cvel
        integer mmax

        integer NL
        parameter (NL=200)
        common/timodel/d(NL),TA(NL),TC(NL),TL(NL),TN(NL),TF(NL),
     1      TRho(NL),
     2      qa(NL),qb(NL),etap(NL),etas(NL),
     3      frefp(NL), frefs(NL)
        real d,TA,TC,TN,TL,TF,TRho,qa,qb,etap,etas,frefp,frefs
        common/depref/refdep
        real refdep
        integer jbdryh
        complex ra, rb
        complex czero, cone
        real om, w
        real va, vb
        real k, ka, kb, gam
        complex k2prarb, k2mrarb

C       COMPLEX TY5
  
        czero = cmplx(0.0,0.0)
        cone  = cmplx(1.0,0.0)
        w = 6.2831853*freq      
        k = w/cvel
        
        va = sqrt(TC(mmax)/Trho(mmax))
        vb = sqrt(TL(mmax)/Trho(mmax))
        ka = w/va
        kb = w/vb
        gam = 2.*(k/kb)**2
c-----
c       jbdryh  = halfspace boundary condition
c           = -1  RIGID
c           =  0  ELASTIC
c           = +1  FREE SURFACE
c-----
c       The Y's are 12 13 14 24 34
c-----
        jbdryh = 0
        if(jbdryh.lt.0)then
            Y(1) = czero
            Y(2) = czero
            Y(3) = czero
            Y(4) = czero
            Y(5) = cone
        else if(jbdryh.eq.0)then
            ra = csqrt(cone*(k*k-ka*ka))
            rb = csqrt(cone*(k*k-kb*kb))
            k2prarb = k*k+ra*rb
            k2mrarb = ((k*k*(ka*ka+kb*kb) -ka*ka*kb*kb))/k2prarb
            Y(1) =   k2mrarb
            Y(2) = - Trho(mmax)*w*w*rb
            Y(3) = (gam*k2mrarb-k*k)*(Trho(mmax)*w*w/k)
            Y(4) =   Trho(mmax)*w*w*ra
            Y(5) =(gam*gam*k2mrarb +k*k*(1-2*gam))*(Trho(mmax)*w*w/k)**2
        else
            Y(1) = cone
            Y(2) = czero
            Y(3) = czero
            Y(4) = czero
            Y(5) = czero
        endif

        return
        end

        subroutine dofill5(amat,lyr,freq,cvel,hh)
c-----
c       equation B.5
c-----
        implicit none
        real amat(5,5)
        integer lyr
        real freq, cvel, hh

        integer NL
        parameter (NL=200)
        common/timodel/h(NL),TA(NL),TC(NL),TL(NL),TN(NL),TF(NL),
     1      TRho(NL),
     2      qa(NL),qb(NL),etap(NL),etas(NL),
     3      frefp(NL), frefs(NL)
        real h,TA,TC,TN,TL,TF,TRho,qa,qb,etap,etas,frefp,frefs
        common/depref/refdep
        real refdep

        integer i,j
        real a,b,c,d,e,f
        real k,w
 
        w = 6.2831853*freq      
        k = w/cvel
        do i=1,5
           do j=1,5
              amat(i,j) = 0.0
           enddo
        enddo
        hh = h(lyr)
        a = - k
        b = 1./TL(lyr)
        c = k*TF(lyr)/TC(lyr)
        d = 1./TC(lyr)
        e = - Trho(lyr)*w*w
        f = - Trho(lyr)*w*w +k*k*(TA(lyr) -TF(lyr)*TF(lyr)/TC(lyr))
        amat(1,2) = d
        amat(1,4) = -b
        amat(2,1) = e
        amat(2,3) = -2*a
        amat(2,5) = -b
        amat(3,2) = -c
        amat(3,4) = a
        amat(4,1) = -f
        amat(4,3) = 2*C
        amat(4,5) = d
        amat(5,2) = -f
        amat(5,4) = e
        return
        end
